require "vacation_rentals_ical_adapters/registry"

class VacationRentalsIcalAdapters::Adapters
  def self.adapter_for(body:, source:)
    registry.find_adapter do |adapter|
      adapter.applicable?(body: body, source: source)
    end
  end

  def self.registry
    @registry ||= VacationRentalsIcalAdapters::Registry.new
  end
  private_class_method :registry

  def self.register(adapter)
    registry.register(adapter)
  end
  private_class_method :register

  require "vacation_rentals_ical_adapters/adapters/ical_adapter"
  require "vacation_rentals_ical_adapters/adapters/xml_arkiane_adapter"
  require "vacation_rentals_ical_adapters/adapters/csv_booked_days_adapter"
  require "vacation_rentals_ical_adapters/adapters/json_availability_map_adapter"

  register(IcalAdapter.new) # must be first
  register(XmlArkianeAdapter.new)
  register(CsvBookedDaysAdapter.new)
  register(JsonAvailabilityMapAdapter.new)
end
