require "active_support/core_ext/hash/conversions"
require "vacation_rentals_ical_adapters/event"
require "vacation_rentals_ical_adapters/ical_converter"

class VacationRentalsIcalAdapters::Adapters::XmlArkianeAdapter
  ARKIANE_XML_HOST = "xml.arkiane.com".freeze
  private_constant :ARKIANE_XML_HOST

  def applicable?(body:, source:)
    URI(source.to_s).host == ARKIANE_XML_HOST
  end

  def to_ical(body)
    events = parse_events(body)
    VacationRentalsIcalAdapters::IcalConverter.to_ical(events)
  end

  private

  def parse_events(body)
    extract_availabilities(body).map do |availability|
      VacationRentalsIcalAdapters::Event.new(
        start_date: availability.fetch("o_debut"),
        end_date: availability.fetch("o_fin")
      )
    end
  end

  def extract_availabilities(body)
    Hash.from_xml(body)
      .to_a
      .flatten
      .last.to_h["LINE"]
      .to_a.flat_map { |el| el["Planning"] }
  end
end
