class VacationRentalsIcalAdapters::IcalConverter
  def self.to_ical(events)
    new(events).to_ical
  end

  attr_reader :events
  private     :events

  def initialize(events)
    @events = events
  end
  private_class_method :new

  def to_ical
    [
      ical_beginning,
      events.map { |event| ical_event_from(event) },
      ical_ending,
    ].reject(&:empty?).join("\n")
  end

  private

  def ical_beginning
    %Q{BEGIN:VCALENDAR
VERSION:2.0}
  end

  def ical_event_from(event)
%Q{BEGIN:VEVENT
DTEND;VALUE=DATE:#{format_date(event.end_date)}
DTSTART;VALUE=DATE:#{format_date(event.start_date)}
END:VEVENT}
  end

  def format_date(date)
    date.strftime("%Y%m%d")
  end

  def ical_ending
    %Q{END:VCALENDAR}
  end
end
